//
//  mod_win32dstr.c
//
//	Copyright 2008 M-Logic, Inc. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "httpd.h"
#include "http_config.h"
#include "http_log.h"
// Windows only
#include <windows.h>

typedef struct {
    int enabled;
} win32dstr_cfg_t;

module AP_MODULE_DECLARE_DATA win32dstr_module;

static void *win32dstr_cfg_create(apr_pool_t *p, server_rec *s)
{
    win32dstr_cfg_t *cfg;
	
	cfg = (win32dstr_cfg_t *)apr_palloc(p, sizeof(*cfg));
    cfg->enabled = 0;
    return cfg;
}

static void win32dstr_error_logger(const char *file, int line, int level, apr_status_t status, const server_rec *s, const request_rec *r, apr_pool_t *pool, const char *error) {
    win32dstr_cfg_t *cfg;

    if (s == NULL || error == NULL) return;
    cfg = (win32dstr_cfg_t *)ap_get_module_config(s->module_config, &win32dstr_module);
    if (cfg == NULL) return;
    if (cfg->enabled)
    {
        OutputDebugString(error);
    }
    return;
}

static void win32dstr_register_hooks(apr_pool_t *p)
{
    ap_hook_error_log(win32dstr_error_logger, NULL, NULL, APR_HOOK_MIDDLE);
}

static const char *win32dstr_cmd_enable(cmd_parms *cmd, void *dummy, int arg)
{
    win32dstr_cfg_t *cfg;

	cfg = (win32dstr_cfg_t *)ap_get_module_config(cmd->server->module_config, &win32dstr_module);
    cfg->enabled = arg;
    return NULL;
}

command_rec win32dstr_cmds[] = 
{
    AP_INIT_FLAG("EnableWin32DStr", win32dstr_cmd_enable, NULL, RSRC_CONF, "Enable Win32DStr"),
    {NULL}
};

module AP_MODULE_DECLARE_DATA win32dstr_module =
{
    STANDARD20_MODULE_STUFF,
    NULL,                       /* per-directory config creator */
    NULL,                       /* dir config merger */
    win32dstr_cfg_create,       /* server config */
    NULL,                       /* server config merger */
    win32dstr_cmds,             /* command table */
    win32dstr_register_hooks,   /* set up other request processing hooks */
};
